# This file is managed by /Users/kobayashi.atsushi/perl5/perlbrew/perls/perl-5.14.2/bin/dim.
{
  dist => 'Data::RuledCluster',
  distdir => 'Data-RuledCluster',
  module => 'Data::RuledCluster',
  template => 'Default'
}
